@@ -55,6 +55,10 @@ opts = [
                default=3,
                help='Maximum attempts to verify an iSCSI connection is '
                     'active, sleeping 1 second between attempts.'),
+    cfg.IntOpt('partprobe_attempts',
+               default=10,
+               help='Maximum number of attempts to try to read the '
+                    'partition.'),
 ]
 
 CONF = cfg.CONF
@@ -140,6 +144,7 @@ def get_disk_identifier(dev):
     http://www.syslinux.org/wiki/index.php/Comboot/chain.c32#mbr:
 
     :param dev: Path for the already populated disk device.
+    :raises OSError: When the hexdump binary is unavailable.
     :returns: The Disk Identifier.
     """
     disk_identifier = utils.execute('hexdump', '-s', '440', '-n', '4',
@@ -377,6 +382,14 @@ def destroy_disk_metadata(dev, node_uuid):
     utils.execute('sgdisk', '-Z', dev, run_as_root=True,
                   use_standard_locale=True)
 
+    try:
+        utils.wait_for_disk_to_become_available(dev)
+    except exception.IronicException as e:
+        raise exception.InstanceDeployFailure(
+            _('Destroying metadata failed on device %(device)s. '
+              'Error: %(error)s')
+            % {'device': dev, 'error': e})
+
     LOG.info("Disk metadata on %(dev)s successfully destroyed for node "
              "%(node)s", {'dev': dev, 'node': node_uuid})
 
@@ -617,7 +630,8 @@ def _get_labelled_partition(device_path, label, node_uuid):
               returns None.
     """
     try:
-        utils.execute('partprobe', device_path, run_as_root=True)
+        utils.execute('partprobe', device_path, run_as_root=True,
+                      attempts=CONF.disk_utils.partprobe_attempts)
 
         # lsblk command
         output, err = utils.execute('lsblk', '-Po', 'name,label', device_path,
@@ -806,6 +820,25 @@ def create_config_drive_partition(node_uuid, device, configdrive):
                 utils.execute('parted', '-a', 'optimal', '-s', '--', device,
                               'mkpart', 'primary', 'fat32', startlimit,
                               endlimit, run_as_root=True)
+            # Parted uses fsync to tell the kernel to sync file io
+            # however on ramdisks in ramfs, this is an explicit no-op.
+            # Explicitly call sync so when the the kernel attempts to read
+            # the partition table from disk, it is less likely that the write
+            # is still in buffer cache pending write to disk.
+            LOG.debug('Explicitly calling sync to force buffer/cache flush.')
+            utils.execute('sync')
+            # Make sure any additions to the partitioning are reflected in the
+            # kernel.
+            LOG.debug('Waiting until udev event queue is empty')
+            utils.execute('udevadm', 'settle')
+            try:
+                utils.execute('partprobe', device, run_as_root=True,
+                              attempts=CONF.disk_utils.partprobe_attempts)
+                # Also verify that the partitioning is correct now.
+                utils.execute('sgdisk', '-v', device, run_as_root=True)
+            except processutils.ProcessExecutionError as exc:
+                LOG.warning('Failed to verify GPT partitioning after creating '
+                            'the configdrive partition: %s', exc)
 
             upd_parts = set(part['number'] for part in list_partitions(device))
             new_part = set(upd_parts) - set(cur_parts)
