@@ -23,7 +23,6 @@ import mock
 from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_serialization import base64
-from oslo_service import loopingcall
 from oslo_utils import imageutils
 import requests
 
@@ -316,7 +315,7 @@ class MakePartitionsTestCase(base.IronicLibTestCase):
 
     def _test_make_partitions(self, mock_exc, boot_option, boot_mode='bios',
                               disk_label=None):
-        mock_exc.return_value = (None, None)
+        mock_exc.return_value = ('', '')
         disk_utils.make_partitions(self.dev, self.root_mb, self.swap_mb,
                                    self.ephemeral_mb, self.configdrive_mb,
                                    self.node_uuid, boot_option=boot_option,
@@ -387,7 +386,7 @@ class MakePartitionsTestCase(base.IronicLibTestCase):
                            'mkpart', 'primary', '', '2561', '3585']
         self.dev = 'fake-dev'
         cmd = self._get_parted_cmd(self.dev) + expected_mkpart
-        mock_exc.return_value = (None, None)
+        mock_exc.return_value = ('', '')
         disk_utils.make_partitions(self.dev, self.root_mb, self.swap_mb,
                                    self.ephemeral_mb, self.configdrive_mb,
                                    self.node_uuid)
@@ -411,7 +410,7 @@ class MakePartitionsTestCase(base.IronicLibTestCase):
                            'swap': swap,
                            'root': root}
         cmd = self._get_parted_cmd(self.dev) + expected_mkpart
-        mock_exc.return_value = (None, None)
+        mock_exc.return_value = ('', '')
         result = disk_utils.make_partitions(
             self.dev, self.root_mb, self.swap_mb, self.ephemeral_mb,
             self.configdrive_mb, self.node_uuid)
@@ -434,7 +433,7 @@ class MakePartitionsTestCase(base.IronicLibTestCase):
                            'swap': swap,
                            'root': root}
         cmd = self._get_parted_cmd(self.dev) + expected_mkpart
-        mock_exc.return_value = (None, None)
+        mock_exc.return_value = ('', '')
         result = disk_utils.make_partitions(
             self.dev, self.root_mb, self.swap_mb, self.ephemeral_mb,
             self.configdrive_mb, self.node_uuid)
@@ -454,7 +453,7 @@ class MakePartitionsTestCase(base.IronicLibTestCase):
                            'swap': 'fake-dev2',
                            'root': 'fake-dev3'}
         cmd = self._get_parted_cmd(self.dev) + expected_mkpart
-        mock_exc.return_value = (None, None)
+        mock_exc.return_value = ('', '')
         result = disk_utils.make_partitions(
             self.dev, self.root_mb, self.swap_mb, self.ephemeral_mb,
             self.configdrive_mb, self.node_uuid)
@@ -465,7 +464,7 @@ class MakePartitionsTestCase(base.IronicLibTestCase):
         self.assertEqual(expected_result, result)
 
 
-@mock.patch.object(utils, 'execute', autospec=True)
+@mock.patch.object(utils, 'execute', autospec=True, return_value=('', ''))
 class DestroyMetaDataTestCase(base.IronicLibTestCase):
 
     def setUp(self):
@@ -479,7 +478,10 @@ class DestroyMetaDataTestCase(base.IronicLibTestCase):
                                     use_standard_locale=True),
                           mock.call('sgdisk', '-Z', 'fake-dev',
                                     run_as_root=True,
-                                    use_standard_locale=True)]
+                                    use_standard_locale=True),
+                          mock.call('fuser', self.dev,
+                                    check_exit_code=[0, 1],
+                                    run_as_root=True)]
         disk_utils.destroy_disk_metadata(self.dev, self.node_uuid)
         mock_exec.assert_has_calls(expected_calls)
 
@@ -514,7 +516,8 @@ class DestroyMetaDataTestCase(base.IronicLibTestCase):
         mock_exec.side_effect = iter(
             [processutils.ProcessExecutionError(description='--force'),
              (None, None),
-             (None, None)])
+             (None, None),
+             ('', '')])
 
         expected_call = [mock.call('wipefs', '--force', '--all', 'fake-dev',
                                    run_as_root=True,
@@ -562,13 +565,7 @@ class PopulateImageTestCase(base.IronicLibTestCase):
         self.assertFalse(mock_dd.called)
 
 
-def _looping_call_done(*args, **kwargs):
-    raise loopingcall.LoopingCallDone()
-
-
-@mock.patch.object(disk_partitioner.DiskPartitioner,
-                   '_wait_for_disk_to_become_available',
-                   _looping_call_done)
+@mock.patch.object(utils, 'wait_for_disk_to_become_available', lambda *_: None)
 @mock.patch.object(disk_utils, 'is_block_device', lambda d: True)
 @mock.patch.object(disk_utils, 'block_uuid', lambda p: 'uuid')
 @mock.patch.object(disk_utils, 'dd', lambda *_: None)
@@ -831,7 +828,7 @@ class WholeDiskPartitionTestCases(base.IronicLibTestCase):
                                                     self.node_uuid)
         self.assertEqual(part_result, result)
         execute_calls = [
-            mock.call('partprobe', self.dev, run_as_root=True),
+            mock.call('partprobe', self.dev, run_as_root=True, attempts=10),
             mock.call('lsblk', '-Po', 'name,label', self.dev,
                       check_exit_code=[0, 1],
                       use_standard_locale=True, run_as_root=True)
@@ -862,7 +859,7 @@ class WholeDiskPartitionTestCases(base.IronicLibTestCase):
                                                     self.node_uuid)
         self.assertIsNone(result)
         execute_calls = [
-            mock.call('partprobe', self.dev, run_as_root=True),
+            mock.call('partprobe', self.dev, run_as_root=True, attempts=10),
             mock.call('lsblk', '-Po', 'name,label', self.dev,
                       check_exit_code=[0, 1],
                       use_standard_locale=True, run_as_root=True)
@@ -880,7 +877,7 @@ class WholeDiskPartitionTestCases(base.IronicLibTestCase):
                                disk_utils._get_labelled_partition, self.dev,
                                self.config_part_label, self.node_uuid)
         execute_calls = [
-            mock.call('partprobe', self.dev, run_as_root=True),
+            mock.call('partprobe', self.dev, run_as_root=True, attempts=10),
             mock.call('lsblk', '-Po', 'name,label', self.dev,
                       check_exit_code=[0, 1],
                       use_standard_locale=True, run_as_root=True)
@@ -894,8 +891,8 @@ class WholeDiskPartitionTestCases(base.IronicLibTestCase):
                                'Failed to retrieve partition labels',
                                disk_utils._get_labelled_partition, self.dev,
                                self.config_part_label, self.node_uuid)
-        mock_execute.assert_called_once_with('partprobe', self.dev,
-                                             run_as_root=True)
+        mock_execute.assert_called_once_with(
+            'partprobe', self.dev, run_as_root=True, attempts=10)
         self.assertEqual(1, mock_log.call_count)
 
     def _test_is_disk_larger_than_max_size(self, mock_execute, blk_out):
@@ -1108,6 +1105,11 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
         mock_execute.assert_has_calls([
             mock.call('sgdisk', '-n', '0:-64MB:0', self.dev,
                       run_as_root=True),
+            mock.call('sync'),
+            mock.call('udevadm', 'settle'),
+            mock.call('partprobe', self.dev, attempts=10, run_as_root=True),
+            mock.call('sgdisk', '-v', self.dev, run_as_root=True),
+
             mock.call('udevadm', 'settle'),
             mock.call('test', '-e', expected_part, attempts=15,
                       check_exit_code=[0], delay_on_retry=True)
@@ -1209,6 +1211,10 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
                                     '-0', run_as_root=True)
         mock_execute.assert_has_calls([
             parted_call,
+            mock.call('sync'),
+            mock.call('udevadm', 'settle'),
+            mock.call('partprobe', self.dev, attempts=10, run_as_root=True),
+            mock.call('sgdisk', '-v', self.dev, run_as_root=True),
             mock.call('udevadm', 'settle'),
             mock.call('test', '-e', expected_part, attempts=15,
                       check_exit_code=[0], delay_on_retry=True)
@@ -1305,10 +1311,17 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
                                self.node_uuid, self.dev, config_url)
 
         mock_get_configdrive.assert_called_with(config_url, self.node_uuid)
-        mock_execute.assert_called_with('parted', '-a', 'optimal', '-s', '--',
-                                        self.dev, 'mkpart', 'primary',
-                                        'fat32', '-64MiB', '-0',
-                                        run_as_root=True)
+        mock_execute.assert_has_calls([
+            mock.call('parted', '-a', 'optimal', '-s', '--',
+                      self.dev, 'mkpart', 'primary',
+                      'fat32', '-64MiB', '-0',
+                      run_as_root=True),
+            mock.call('sync'),
+            mock.call('udevadm', 'settle'),
+            mock.call('partprobe', self.dev, attempts=10, run_as_root=True),
+            mock.call('sgdisk', '-v', self.dev, run_as_root=True),
+        ])
+
         self.assertEqual(2, mock_list_partitions.call_count)
         mock_fix_gpt_partition.assert_called_with(self.dev, self.node_uuid)
         mock_is_disk_gpt.assert_called_with(self.dev, self.node_uuid)
